@@ -30,8 +30,7 @@ use Perl6::Junction qw(any);
 use Date::Calc::Object qw(:all);
 use File::Copy::Recursive qw(dircopy fcopy);
 
-# CA inherits from GConfModule
-use base 'EBox::GConfModule';
+use base qw(EBox::GConfModule EBox::Model::ModelProvider EBox::Module::Service);
 
 use EBox::CA::DN;
 use EBox::Gettext;
@@ -43,6 +42,7 @@ use EBox::Exceptions::Internal;
 use EBox::Exceptions::DataMissing;
 use EBox::Exceptions::DataInUse;
 use EBox;
+use EBox::CA::Certificates;
 
 use constant TEMPDIR     => EBox::Config->tmp(); # "/tmp/";
 use constant OPENSSLPATH => "/usr/bin/openssl";
@@ -112,6 +112,7 @@ sub _create
 	my $class = shift;
 	my $self = $class->SUPER::_create(name => 'ca',
 					  domain => 'ebox-ca',
+					  printableName => __('Certification Authority'),
 #					  title => __n('Certification Authority'),
 					  @_);
 
@@ -151,11 +152,26 @@ sub _create
 # Returns:
 #
 #       string - the gettext domain
-
+#
 sub domain
-  {
+{
     return 'ebox-ca';
-  }
+}
+
+# Method: modelClasses
+#
+#       Return the model classes
+#
+# Returns:
+#
+#       array - the model classes
+#
+sub modelClasses
+{
+    return [
+        'EBox::CA::Model::Certificates',
+    ];
+}
 
 # Method: isCreated
 #
@@ -179,6 +195,28 @@ sub isCreated
 
   }
 
+# Method: isAvailable
+#
+#       Check whether the Certification Infrastructure is available or not
+#
+# Returns:
+#
+#       boolean - True, if the Certification Infrastructure is available.
+#                 Undef, otherwise.
+#
+sub isAvailable
+{
+    my ($self) = @_;
+
+    return 0 unless $self->isCreated();
+    my $currentState = $self->currentCACertificateState();
+    if ( $currentState =~ m/[RE]/) {
+        return 0;
+    } else {
+        return 1;
+    }
+}
+
 # Method: createCA
 #
 #       Create a Certification Authority with a self-signed certificate
@@ -308,6 +346,9 @@ sub createCA {
   # Set the CA certificate expiration date
   $self->{caExpirationDate} = $self->_obtain(CACERT, 'endDate');
 
+  # Generate CRL
+  $self->generateCRL();
+
   # Logging the action
 #  logAdminNow($self->name, "createCA",
 #	      "orgName=". $self->{dn}->attribute('orgName') . ",days=" . $args{days}
@@ -679,7 +720,7 @@ sub CAPublicKey {
 #                        a private key file in the CA (Optional)
 #
 #       requestFile - path to save the new certificate request
-#                    (Optional) 
+#                    (Optional)
 #       certFile - path to store the new certificate file (Optional)
 #
 #
@@ -726,22 +767,18 @@ sub issueCertificate
   }
 
   # Check the expiration date unless issuing the CA
+  my $userExpDay = undef;
   if (not defined($args{certFile}) or
       $args{certFile} ne CACERT) {
-    my $userExpDay = undef;
     if ( defined($days) ) {
       $userExpDay = Date::Calc::Object->now();
       $userExpDay += [0, 0, $days, 0, 0, 0];
     } elsif (defined $args{endDate}) {
       $userExpDay = $args{endDate};
     }
-
-    $self->{caExpirationDate} = $self->_obtain(CACERT, "endDate")
-      unless defined($self->{caExpirationDate});
-
-    if ( $userExpDay gt $self->{caExpirationDate} ) {
-      throw EBox::Exceptions::External(__("Expiration date later than CA certificate expiration date"));
-    }
+    $userExpDay = $self->_isLaterThanCA($userExpDay);
+    # Set to undef since we are using userExpDay as endDate
+    $days = undef;
   }
 
 
@@ -803,10 +840,10 @@ sub issueCertificate
   }
 
   $output = $self->_signRequest( userReqFile  => $userReq,
-				 days         => $days,
-				 userCertFile => $certFile,
+                                 days         => $days,
+                                 userCertFile => $certFile,
 				 selfsigned   => $selfSigned,
-				 endDate      => $args{endDate}
+				 endDate      => $userExpDay,
 			       );
 
   # Check if something goes wrong
@@ -845,6 +882,7 @@ sub issueCertificate
 #       caKeyPassword - the CA passpharse (Optional)
 #       certFile - the Certificate to revoke (Optional)
 #       force    - Force the revokation (Optional)
+#       renew    - This revoke is due to a renew (Optional)
 #
 # Returns:
 #
@@ -899,12 +937,12 @@ sub revokeCertificate {
   if ( $args{force} ) {
     $self->_freeCert($commonName, $isCACert);
   }
-  elsif ( $self->_certsInUse($commonName, $isCACert) ) {
+  elsif ( not $args{renew} and $self->_certsInUse($commonName, $isCACert) ) {
     throw EBox::Exceptions::DataInUse("Certificate $commonName in use");
   }
 
   # TODO: Different kinds of revokations (CACompromise,
-  # keyCompromise...) 
+  # keyCompromise...)
   # We can set different arguments regard to revocation reason
   my $cmd = "ca";
   $self->_commonArgs("ca", \$cmd);
@@ -915,7 +953,7 @@ sub revokeCertificate {
   $cmd .= "-crl_reason $reason " if (defined($reason));
 
   # Tell openssl to revoke
-  $ENV{'PASS'} = $self->{caKeyPassword} 
+  $ENV{'PASS'} = $self->{caKeyPassword}
     if (defined($self->{caKeyPassword}));
   my ($retValue, $output) = $self->_executeCommand(COMMAND => $cmd);
   delete ($ENV{'PASS'});
@@ -924,33 +962,8 @@ sub revokeCertificate {
   throw EBox::Exceptions::External($self->_filterErrorFromOpenSSL($output))
     if ($retValue eq "ERROR");
 
-  # Generate a new Certification Revocation List (For now in the same
-  # method...)
-
-  # Localtime
-  (my $day, my $month, my $year) = Date::Calc::Today();
-
-  my $date = sprintf("%04d-%02d-%02d", $year+1900, $month+1, $day);
-
-  $cmd= "ca";
-  $self->_commonArgs("ca", \$cmd);
-  $cmd .= "-gencrl ";
-  if ( defined($self->{caKeyPassword}) ){
-    $cmd .= "-passin env:PASS ";
-  }
-  $cmd .= "-out " . CRLDIR . $date . "-crl.pem";
-
-  $ENV{'PASS'} = $self->{caKeyPassword}
-    if (defined($self->{caKeyPassword}));
-  ($retValue, $output) = $self->_executeCommand(COMMAND => $cmd);
-  delete ($ENV{'PASS'});
-
-  throw EBox::Exceptions::External($self->_filterErrorFromOpenSSL($output))
-    if ($retValue eq "ERROR");
-
-  # Set the link to the last
-  unlink (LASTESTCRL) if ( -e LASTESTCRL );
-  symlink ( CRLDIR . $date . "-crl.pem", LASTESTCRL );
+  # Generate a new Certification Revocation List
+  $self->generateCRL();
 
   # Mark this module as changed
   $self->setAsChanged();
@@ -964,6 +977,43 @@ sub revokeCertificate {
 
 }
 
+# Method: generateCRL
+#
+#       Generate a new Certification Revocation List. It will create a
+#       new one with today date and link to the latest.
+#
+sub generateCRL
+{
+    my ($self) = @_;
+
+    # Localtime
+    (my $year, my $month, my $day) = Date::Calc::Today();
+
+    my $date = sprintf("%04d-%02d-%02d", $year, $month, $day);
+
+    my $cmd= "ca";
+    $self->_commonArgs("ca", \$cmd);
+    $cmd .= "-gencrl ";
+    if ( defined($self->{caKeyPassword}) ) {
+        $cmd .= "-passin env:PASS ";
+    }
+    $cmd .= "-out " . CRLDIR . $date . "-crl.pem";
+
+    $ENV{'PASS'} = $self->{caKeyPassword}
+      if (defined($self->{caKeyPassword}));
+    my ($retValue, $output) = $self->_executeCommand(COMMAND => $cmd);
+    delete ($ENV{'PASS'});
+
+    throw EBox::Exceptions::External($self->_filterErrorFromOpenSSL($output))
+      if ($retValue eq "ERROR");
+
+    # Set the link to the last
+    unlink (LASTESTCRL) if ( -e LASTESTCRL );
+    symlink ( CRLDIR . $date . "-crl.pem", LASTESTCRL );
+
+}
+
+
 # Method: listCertificates
 #
 #       List the certificates that are ready on the system sorted
@@ -1025,10 +1075,10 @@ sub listCertificates
       $element{'serialNumber'} = $line[SERIAL_IDX];
 
       # Set paramaters regarding to certificate validity
-      if ($element{'state'} eq 'V') {
+      if (($element{'state'} eq 'V') or ($element{'state'} eq 'E')) {
 	$element{'expiryDate'} = $self->_parseDate($line[EXPIRE_DATE_IDX]);
 	$element{'isCACert'} = $self->_isCACert($element{'dn'});
-      } else {
+      } elsif ( $element{'state'} eq 'R' ) {
 	my $field = $line[REV_DATE_REASON_IDX];
 	my ($revDate, $reason) = split(',', $field);
 	$element{'revokeDate'} = $self->_parseDate($revDate);
@@ -1103,7 +1153,7 @@ sub getCertificateMetadata
 					 );
     } elsif ( scalar(keys %args) > 1 ) {
       throw EBox::Exceptions::Internal("Only one parameter is necessary");
-    } elsif ( defined($args{'dn'}) 
+    } elsif ( defined($args{'dn'})
 	      and not $args{'dn'}->isa('EBox::CA::DN') ) {
       throw EBox::Exceptions::Internal("dn parameter should be an instance of EBox::CA::DN");
     }
@@ -1290,8 +1340,7 @@ sub removePrivateKey
 #
 
 sub renewCertificate
-  {
-
+{
     my ($self, %args) = @_;
 
     if (not defined($args{endDate}) ) {
@@ -1309,20 +1358,16 @@ sub renewCertificate
       if (defined($args{endDate}) and not UNIVERSAL::isa($args{endDate}, "Date::Calc"));
 
     # Check the expiration date unless issuing the CA cert
-    unless (defined($args{certFile}) or $args{certFile} eq CACERT) {
-      my $userExpDay = undef;
+    my $userExpDay = undef;
+    unless (defined($args{certFile}) and $args{certFile} eq CACERT) {
       if ( defined($args{days}) ) {
 	$userExpDay = Date::Calc::Object->now() + [0, 0, +$args{days}, 0, 0, 0];
       } elsif ( defined($args{endDate}) ) {
 	$userExpDay = $args{endDate};
       }
-
-      $self->{caExpirationDate} = $self->_obtain(CACERT, "endDate")
-	unless defined($self->{caExpirationDate});
-
-      if ( $userExpDay gt $self->{caExpirationDate} ) {
-	throw EBox::Exceptions::External(__("Expiration date later than CA certificate expiration date"));
-      }
+      $userExpDay = $self->_isLaterThanCA($userExpDay);
+      # Set to undef since we are using userExpDay as endDate
+      $args{days} = undef;
     }
 
     throw EBox::Exceptions::DataMissing(data => __('Common Name')
@@ -1358,7 +1403,7 @@ sub renewCertificate
       $dnFieldHasChanged = "1";
       $userDN->attribute('countryName', $args{countryName});
     }
-    if (defined($args{stateName}) 
+    if (defined($args{stateName})
 	and $args{stateName} ne $userDN->attribute('stateName')) {
       $dnFieldHasChanged = "1" ;
       $userDN->attribute('stateName', $args{stateName});
@@ -1385,6 +1430,7 @@ sub renewCertificate
 					  certFile      => $userCertFile,
 					  caKeyPassword => $args{caKeyPassword},
 					  force         => $args{force},
+                                          renew         => "yes",
 					 );
 
     if (defined($retVal) ) {
@@ -1441,7 +1487,7 @@ sub renewCertificate
 					selfsigned   => $selfsigned,
 					createSerial => 0,
 # Not in OpenSSL 0.9.7			newSubject   => $newSubject,
-					endDate      => $args{endDate}
+					endDate      => $userExpDay,
 				      );
 
       if (defined($output) ) {
@@ -1478,9 +1524,18 @@ sub renewCertificate
 #		"cn=" . $args{commonName} . ",days=" . $args{days}
 #	       );
 
-    return $newCertFile;
+    # Tells other modules the following certs have been renewed
+    my $isCACert = 0;
+    $isCACert = 1 if (defined($args{certFile}) and $args{certFile} eq CACERT);
+    my $global = EBox::Global->getInstance();
+    my @mods = @{$global->modInstancesOfType('EBox::CA::Observer')};
+    foreach my $mod (@mods) {
+        $mod->certificateRenewed($userDN->attribute('commonName'), $isCACert);
+    }
+    EBox::CA::Certificates->certificateRenewed($userDN->attribute('commonName'), $isCACert);
 
-  }
+    return $newCertFile;
+}
 
 # Method: updateDB
 #
@@ -1511,7 +1566,7 @@ sub updateDB
       $self->{caKeyPassword} = $caKeyPassword;
     }
 
-    my @expiredCertsBefore = $self->listCertificates(state => 'E');
+    my @expiredCertsBefore = @{$self->listCertificates(state => 'E')};
 
     my $cmd = "ca";
     $self->_commonArgs("ca", \$cmd);
@@ -1520,18 +1575,21 @@ sub updateDB
       $cmd .= "-passin env:PASS ";
     }
 
-    $ENV{'PASS'} = $self->{caKeyPassword};
+    $ENV{'PASS'} = $self->{caKeyPassword} if defined($self->{caKeyPassword});
     my ($retVal, $output) = $self->_executeCommand( COMMAND => $cmd );
     delete( $ENV{'PASS'} );
 
-    my @expiredCertsAfter = $self->listCertificates(state => 'E');
-    # Finding elements in one array but not another
-    # T. Christiansen & N. Torkington, Perl Cookbook 2nd edition, pg. 126
+    my @expiredCertsAfter = @{$self->listCertificates(state => 'E')};
     my %seen;
-    @seen {@expiredCertsAfter} = ();
-    delete @seen {@expiredCertsBefore};
+    my @diff;
+    foreach my $item (@expiredCertsBefore) {
+        $seen{$item->{serialNumber}} = 1;
+    }
+    foreach my $item (@expiredCertsAfter) {
+        next if ( $seen{$item->{serialNumber}} );
+        push(@diff, $item);
+    }
 
-    my @diff = keys %seen;
     # Tells other modules the following certs have expired
     my $global = EBox::Global->getInstance();
     my @mods = @{$global->modInstancesOfType('EBox::CA::Observer')};
@@ -1540,6 +1598,7 @@ sub updateDB
 	$mod->certificateExpired($cert->{dn}->attribute('commonName'),
 				 $cert->{isCACert});
       }
+      EBox::CA::Certificates->certificateExpired($cert->{dn}->attribute('commonName'), $cert->{isCACert});
     }
 
     if ($retVal eq "ERROR") {
@@ -1627,10 +1686,6 @@ sub getCurrentCRL
   }
 
 
-# _regenConfig is not longer needed 'cause this module doesn't manage a daemon
-
-# The method summary is not neccessary since it is not a network service
-
 # Method: menu
 #
 #       Add CA module to eBox menu
@@ -1639,16 +1694,23 @@ sub getCurrentCRL
 #
 #       root - the <EBox::Menu::Root> where to leave our items
 #
-
 sub menu
-  {
-
+{
   my ($self, $root) = @_;
 
-  $root->add(new EBox::Menu::Item('url'  => 'CA/Index',
-				  'text' => __('Certificate Manager')));
+  my $folder = new EBox::Menu::Folder('name' => 'CA',
+                    'text' => $self->printableName(),
+                    'separator' => __('Infrastructure'),
+                    'order' => 440);
 
-  }
+  $folder->add(new EBox::Menu::Item('url'  => 'CA/Index',
+                                    'text' => __('General')));
+
+  $folder->add(new EBox::Menu::Item('url'  => 'CA/View/Certificates',
+                                    'text' => __('Services Certificates')));
+
+  $root->add($folder);
+}
 
 # Method: dumpConfig
 #
@@ -1706,9 +1768,41 @@ sub restoreConfig
 
   }
 
+# Method: showModuleStatus
+#
+#   Indicate to ServiceManager if the module must be shown in Module
+#   status configuration.
+#
+# Overrides:
+#   EBox::Module::Service::showModuleStatus
+#
+sub showModuleStatus
+{
+    # we don't want it to appear in module status
+    return undef;
+}
+
+# Method: _supportActions
+#
+#   Overrides <EBox::Module::ServiceBase>
+#
+#   This method determines if the service will have a button to start/restart
+#   it in the module status widget. By default services will have the button
+#   unless this method is overriden to return undef
+sub _supportActions
+{
+    return undef;
+}
+
 # Group: Private methods
 
-# Obtain the public key given the private key 
+# Method: _setConf # # Overrides: # # <EBox::Module::Service::_setConf> #
+sub _setConf { my ($self) = @_;
+
+    EBox::CA::Certificates->genCerts();
+}
+
+# Obtain the public key given the private key
 #
 # Return public key path if it is correct or undef if password is
 # incorrect
@@ -1729,7 +1823,7 @@ sub _getPubKey # (privKeyFile, pubKeyFile, password?)
       $cmd .= "-passin env:PASS ";
     }
 
-    $ENV{'PASS'} = $password 
+    $ENV{'PASS'} = $password
       if (defined($password));
     my ($retVal) = $self->_executeCommand(COMMAND => $cmd);
     delete( $ENV{'PASS'} );
@@ -1747,7 +1841,7 @@ sub _certFile {
   my $serial = shift;
 
   my $file = CATOPDIR . "newcerts/${serial}.pem";
-  
+
   return $file
 
 }
@@ -1796,7 +1890,6 @@ sub _createRequest # (reqFile, genKey, privKey, keyPassword, dn, needPass?)
     $self->_commonArgs('req', \$cmd);
 
     $cmd .= '-new ';
-    $self->_commonArgs('req', \$cmd);
     if ($args{genKey}) {
       $cmd .= qq{-keyout '$args{privKey}' };
       if (defined($args{keyPassword})) {
@@ -1880,7 +1973,8 @@ sub _signRequest # (userReqFile, days, userCertFile?, policy?, selfsigned?,
     }
     $cmd .= "-outdir " . CERTSDIR . " ";
     $cmd .= "-out \'$args{userCertFile}\' " if defined($args{userCertFile});
-    $cmd .= "-extensions v3_ca " if ( EXTENSIONS_V3);
+    $cmd .= '-extensions v3_ca ' if ( EXTENSIONS_V3 and $args{selfsigned});
+    $cmd .= '-extensions usr_cert ' if ( EXTENSIONS_V3 and not $args{selfsigned});
     # Only available in OpenSSL 0.9.8
     $cmd .= "-selfsign " if ($args{selfsigned});
     $cmd .= "-policy $policy ";
@@ -1933,8 +2027,8 @@ sub _signSelfSignRequest # (userReqFile, days?, userCertFile,
     }
     $cmd .= "-key $args{keyFile} ";
     $cmd .= "-in \'$args{userReqFile}\' ";
-    
-    $ENV{'PASS'} = $self->{caKeyPassword} 
+
+    $ENV{'PASS'} = $self->{caKeyPassword}
       if (defined($self->{caKeyPassword}));
     my ($retVal, $output) = $self->_executeCommand(COMMAND => $cmd);
     delete ( $ENV{'PASS'} );
@@ -1949,7 +2043,7 @@ sub _signSelfSignRequest # (userReqFile, days?, userCertFile,
 # Taken the OpenSSL command (req, x509, rsa...)
 # and add to the args the common arguments to all openssl commands
 # For now, to req and ca commands, it adds config file and batch mode
-# 
+#
 sub _commonArgs # (cmd, args)
   {
 
@@ -2008,11 +2102,11 @@ sub _obtain # (certFile, attribute)
     } elsif ($attribute eq 'serial' ) {
       return $output;
     } elsif ($attribute eq 'endDate' ) {
-      my ($monthStr, $day, $hour, $min, $sec, $yyyy) = 
+      my ($monthStr, $day, $hour, $min, $sec, $yyyy) =
 	($output =~ /(.+) (\d+) (\d+):(\d+):(\d+) (\d+) (.+)/);
 
       $monthStr =~ s/ +//g;
-      my $dateObj = Date::Calc->new($yyyy, 
+      my $dateObj = Date::Calc->new($yyyy,
 				    Decode_Month($monthStr),
 				    $day, $hour, $min, $sec);
       return $dateObj;
@@ -2041,7 +2135,7 @@ sub _parseDate
   }
 
 # A private method to flat a Date::Calc::Object to a OpenSSL form like
-# YYMMDDHHMMSSZ 
+# YYMMDDHHMMSSZ
 sub _flatDate # (date)
   {
     my ($self, $date) = @_;
@@ -2077,7 +2171,7 @@ sub _createSerial
   }
 
 # Get the maximum number of days to apply to a certificate expiration.
-# This is the Year 2038 Bug, explained here -> 
+# This is the Year 2038 Bug, explained here ->
 # http://www.mail-archive.com/openssl-users@openssl.org/msg45886.html
 sub _maxDays
   {
@@ -2095,7 +2189,7 @@ sub _maxDays
 
 # Print the row for CA cert in index.txt file (to fuck them up)
 sub _putInIndex # (EBox::CA::DN dn, String certFile, String
-                  # serialNumber) 
+                  # serialNumber)
   {
 
     my ($self, %args) = @_;
@@ -2131,7 +2225,7 @@ sub _isCACert # (EBox::CA::DN dn)
 
 # Write down the next serial to serial file, given a certificate file
 # Only useful under OpenSSL 0.9.7, later on fixed
-sub _writeDownNextSerial # (certFile) 
+sub _writeDownNextSerial # (certFile)
   {
 
     my ($self, $certFile) = @_;
@@ -2211,6 +2305,7 @@ sub _certsInUse # (cn?, isCACert?)
 	return 1;
     }
     }
+    return 1 if EBox::CA::Certificates->certificateRevoked($cn, $isCACert);
 
     return undef;
 
@@ -2233,6 +2328,7 @@ sub _freeCert # (cn?, isCACert?)
     foreach my $obs (@observers) {
         $obs->freeCertificate($cn);
     }
+    EBox::CA::Certificates->freeCertificate($cn);
 
   }
 
@@ -2276,6 +2372,31 @@ sub _setLogAdminActions
 
 }
 
+# Check if a date is later than CA certificate date
+# Returns the value to set the expiration date
+sub _isLaterThanCA # (date)
+{
+    my ($self, $date) = @_;
+
+    $self->{caExpirationDate} = $self->_obtain(CACERT, "endDate")
+      unless defined($self->{caExpirationDate});
+
+    # gt compares date and time
+    if ( $date gt $self->{caExpirationDate} ) {
+        # != compares date
+        if ( $date > $self->{caExpirationDate} ) {
+            throw EBox::Exceptions::External(
+                __('Expiration date later than CA certificate expiration date')
+               );
+        } else {
+            EBox::warn('Defining CA expiration date for a new certificate');
+            return $self->{caExpirationDate};
+        }
+    }
+    return $date;
+
+}
+
 
 ## OpenSSL execution environment provided by OpenCA::OpenSSL
 ## through OpenCA application
@@ -2289,14 +2410,14 @@ sub _setLogAdminActions
 ## apply to all code found in this distribution, be it the RC4, RSA,
 ## lhash, DES, etc., code; not just the SSL code.  The documentation
 ## included with this distribution is covered by the same copyright terms
-## 
+##
 ## // Copyright remains Massimiliano Pala's, and as such any Copyright notices
 ## in the code are not to be removed.
 ## If this package is used in a product, Massimiliano Pala should be given
 ## attribution as the author of the parts of the library used.
 ## This can be in the form of a textual message at program startup or
 ## in documentation (online or textual) provided with the package.
-## 
+##
 ## Redistribution and use in source and binary forms, with or without
 ## modification, are permitted provided that the following conditions
 ## are met:
@@ -2309,7 +2430,7 @@ sub _setLogAdminActions
 ##    must display the following acknowledgement:
 ## //   "This product includes OpenCA software written by Massimiliano Pala
 ## //    (madwolf@openca.org) and the OpenCA Group (www.openca.org)"
-## 4. If you include any Windows specific code (or a derivative thereof) from 
+## 4. If you include any Windows specific code (or a derivative thereof) from
 ##    some directory (application code) you must include an acknowledgement:
 ##    "This product includes OpenCA software (www.openca.org)"
 ##
